<?php
declare(strict_types=1);

namespace Catequil\Invoice\Repository;

use Catequil\Invoice\Model\InvoiceInterface;
use Doctrine\ORM\EntityRepository;
use function intval;
use function preg_match;

/**
 * Class InvoiceRepository
 *
 * @package Catequil\Invoice\Repository
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-invoice
 *
 * @method InvoiceInterface|null find(int $id);
 */
final class InvoiceRepository extends EntityRepository
{
    public function findByInvoiceNo(string $documentType, int $invoiceNo): ?InvoiceInterface
    {
        return $this->createQueryBuilder('i')
                ->where('i.documentType = ?1')
                ->andWhere('i.invoiceNo = ?2')
                ->orderBy('i.createdAt', 'DESC')
                ->setMaxResults(1)
                ->getQuery()
                ->setParameter(1, $documentType)
                ->setParameter(2, $invoiceNo)
                ->getResult()[0] ?? null;
    }

    public function findByLabel(string $label): ?InvoiceInterface
    {
        if (preg_match('/^([A-Z]+)-?(\d+)$/', $label, $matches)) {
            return $this->findByInvoiceNo($matches[1], intval($matches[2]));
        }
        return null;
    }
}