<?php
declare(strict_types=1);

namespace Catequil\Invoice\Service;

use Catequil\Invoice\Model\Invoice;
use Catequil\Invoice\Model\InvoiceInterface;
use Catequil\Invoice\Repository\InvoiceRepository;
use Catequil\Webshop\Model\WebshopOrder;
use Catequil\Webshop\Service\WebshopServiceInterface;
use Doctrine\ORM\EntityManagerInterface;
use Laminas\Hydrator\HydratorInterface;
use function array_map;
use function array_walk;
use function intval;
use function preg_replace;

/**
 * Class InvoiceService
 *
 * @package Catequil\Invoice\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-invoice
 */
final class InvoiceService implements InvoiceServiceInterface
{
    private EntityManagerInterface $em;
    private HydratorInterface $hydrator;
    private WebshopServiceInterface $webshops;

    /**
     * @param EntityManagerInterface $em
     * @param HydratorInterface $hydrator
     * @param WebshopServiceInterface $webshops
     */
    public function __construct(
        EntityManagerInterface $em,
        HydratorInterface $hydrator,
        WebshopServiceInterface $webshops
    ) {
        $this->em = $em;
        $this->hydrator = $hydrator;
        $this->webshops = $webshops;
    }

    public function invoices(): InvoiceRepository
    {
        /** @noinspection PhpIncompatibleReturnTypeInspection */
        return $this->em->getRepository(Invoice::class);
    }

    /**
     * @param array $data
     * @return InvoiceInterface
     */
    public function uploadFromGath(array $data): InvoiceInterface
    {
        $webshop = $this->webshops->findByDomain($data['WebShopSiteName']);
        $orders = array_map(
            fn (int $orderId) => $this->webshops->orders()->findOrCreate($webshop, $orderId),
            array_map(
                fn (string $orderId) => intval(preg_replace('/^\D+/', '', $orderId)),
                $data['WooIdNarudzbe'] ?? []
            )
        );

        // spremamo račun u bazu
        $invoice = $this->hydrator->hydrate($data, new Invoice());
        array_walk($orders, [$invoice, 'addOrder']);
        $this->em->persist($invoice);
        $this->em->flush();

        return $invoice;
    }
}
